"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IssueService = void 0;
const common_1 = require("@nestjs/common");
const origin_247_certificate_1 = require("@energyweb/origin-247-certificate");
const EnergyTransferRequest_repository_1 = require("./repositories/EnergyTransferRequest.repository");
const cqrs_1 = require("@nestjs/cqrs");
const EnergyTransferRequest_1 = require("./EnergyTransferRequest");
const lodash_1 = require("lodash");
const validate_batch_1 = require("./batch/validate.batch");
const configuration_1 = require("./batch/configuration");
let IssueService = class IssueService {
    constructor(certificateService, etrRepository, batchConfiguration, eventBus) {
        this.certificateService = certificateService;
        this.etrRepository = etrRepository;
        this.batchConfiguration = batchConfiguration;
        this.eventBus = eventBus;
    }
    async issueTask() {
        const etrs = await this.etrRepository.findByState(EnergyTransferRequest_1.State.IssuanceAwaiting);
        const etrGroups = lodash_1.chunk(etrs, this.batchConfiguration.issueBatchSize);
        for (const group of etrGroups) {
            await this.issueCertificates(group);
        }
    }
    async issueCertificates(etrs) {
        etrs.forEach((etr) => etr.issuanceStarted());
        await this.etrRepository.saveManyInTransaction(etrs);
        try {
            const results = await this.certificateService.batchIssue(etrs.map((etr) => {
                const attrs = etr.toPublicAttrs();
                return {
                    energyValue: attrs.volume,
                    userId: attrs.sellerAddress,
                    toAddress: attrs.sellerAddress,
                    deviceId: attrs.certificateData.generatorId,
                    fromTime: new Date(attrs.certificateData.fromTime),
                    toTime: new Date(attrs.certificateData.toTime),
                    metadata: attrs.certificateData.metadata
                };
            }));
            etrs.forEach((etr, index) => etr.issuanceFinished(results[index]));
            etrs.forEach((etr) => etr.persisted());
            await this.etrRepository.saveManyInTransaction(etrs);
            this.eventBus.publish(new validate_batch_1.AwaitingValidationEvent());
        }
        catch (e) {
            etrs.forEach((etr) => etr.issuanceError(e.message));
            await this.etrRepository.saveManyInTransaction(etrs);
        }
    }
};
IssueService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(origin_247_certificate_1.CERTIFICATE_SERVICE_TOKEN)),
    __param(1, common_1.Inject(EnergyTransferRequest_repository_1.ENERGY_TRANSFER_REQUEST_REPOSITORY)),
    __param(2, common_1.Inject(configuration_1.BATCH_CONFIGURATION_TOKEN)),
    __metadata("design:paramtypes", [origin_247_certificate_1.CertificateService, Object, Object, cqrs_1.EventBus])
], IssueService);
exports.IssueService = IssueService;
//# sourceMappingURL=issue.service.js.map